(*
 * Copyright (c) 2002 Andr di Geronimo (adg@home.se)
 *)
 
unit FConvTex;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, FileCtrl,
  O_jkt, O_jkc, FGobDlg;

const
  OPEN_PAL_FILTER = 'All supported files (*.cmp, *.gob, *.goo)|*.cmp;*.gob;*.goo|' +
                    'Jedi Knight Container files (*.gob, *.goo)|*.gob;*.goo|' +
                    'Jedi Knight Palette files (*.cmp)|*.cmp';
  OPEN_GOB_FILTER = 'Jedi Knight Container files (*.gob, *.goo)|*.gob;*.goo|';
  INI_DIR      = 'ConvTexDir';
  INI_GOB      = 'ConvTexGob';
  INI_OUTPUTDIR = 'ConvTexOutputDir';

type
  TfrmConvTex = class(TForm)
    btnConvert: TButton;
    edtPal: TEdit;
    Label3: TLabel;
    pnlStatus: TPanel;
    Label1: TLabel;
    edtOutDir: TEdit;
    btnPalette: TSpeedButton;
    gbxInput: TGroupBox;
    rbtDir: TRadioButton;
    edtDir: TEdit;
    rbtContainer: TRadioButton;
    edtContainer: TEdit;
    btnContainer: TSpeedButton;
    Label2: TLabel;
    rbtJPEG: TRadioButton;
    rbtTGA: TRadioButton;
    btnDir: TSpeedButton;
    btnOutDir: TSpeedButton;
    procedure btnConvertClick(Sender: TObject);
    procedure btnPaletteClick(Sender: TObject);
    procedure btnContainerClick(Sender: TObject);
    procedure rbtDirClick(Sender: TObject);
    procedure btnDirClick(Sender: TObject);
    procedure btnOutDirClick(Sender: TObject);
  private
    FaPalette : TCMPPalette;
    procedure ConvertFromDirectory(const sDir : string);
    procedure ConvertFromContainer(const sFilename : string);
  public
    class function ShowDlg(stlSettings : TStringList) : boolean;
  end;

implementation

{$R *.dfm}

uses
  ShellApi;

{------------------------------------------------------------------------------}
class function TfrmConvTex.ShowDlg(stlSettings : TStringList): boolean;
begin
  with TfrmConvTex.Create(nil) do
  try
    edtDir.Text := stlSettings.Values[INI_DIR];
    edtContainer.Text := stlSettings.Values[INI_GOB];
    edtOutDir.Text := stlSettings.Values[INI_OUTPUTDIR];
    FaPalette := DEFAULT_PALETTE;
    rbtDirClick(nil);
    Result := ShowModal = mrOK;
    stlSettings.Values[INI_DIR] := edtDir.Text;
    stlSettings.Values[INI_GOB] := edtContainer.Text;
    stlSettings.Values[INI_OUTPUTDIR] := edtOutDir.Text;
  finally
    Free;
  end;
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.btnConvertClick(Sender: TObject);
begin
  btnConvert.Enabled := FALSE;
  Screen.Cursor := crHourGlass;
  try
    if rbtDir.Checked then
      ConvertFromDirectory(edtDir.Text)
    else if rbtContainer.Checked then
      if FileExists(edtContainer.Text) then
        ConvertFromContainer(edtContainer.Text);
  finally
    Screen.Cursor := crDefault;
    btnConvert.Enabled := TRUE;
  end;
end;


{------------------------------------------------------------------------------}
procedure TfrmConvTex.btnPaletteClick(Sender: TObject);
var
  sFilename : string;
  sExt : string;
  jkc : TJKContainer;
  sItemFilename : string;
  recHead : TCMPHeader;
  stm : TMemoryStream;
begin
  with TOpenDialog.Create(nil) do
  try
    Filter := OPEN_PAL_FILTER;
    if Execute then
      sFilename := Filename;
  finally
    Free;
  end;

  if sFilename = '' then
    Exit; //!!

  sExt := LowerCase(ExtractFileExt(sFilename));
  if sExt = '.cmp' then begin

    // read palette form plain file
    with TFileStream.Create(sFilename, fmOpenRead) do
    try
      Position := 0;
      ReadBuffer(recHead, SizeOf(recHead));
      ReadBuffer(FaPalette, SizeOf(FaPalette));
    finally
      Free;
    end;
    edtPal.Text := sFilename;

  end
  else if (sExt = '.gob') or (sExt = '.goo') then begin

    jkc := TJKContainer.Create;
    try
      jkc.LoadFromFile(sFilename);
      sItemFilename := TfrmDlgGob.ShowDlg(jkc, 'misc\cmp\');
      if sItemFilename <> '' then begin
        stm := TMemoryStream.Create;
        try
          // read palette from file in container 
          jkc.ExtractToStream(stm, sItemFilename);
          stm.Position := 0;
          stm.ReadBuffer(recHead, SizeOf(recHead));
          stm.ReadBuffer(FaPalette, SizeOf(FaPalette));
        finally
          stm.Free;
        end;
        edtPal.Text := sFilename + '>>' + sItemFilename;
      end;
    finally
      jkc.Free;
    end;

  end;
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.btnContainerClick(Sender: TObject);
begin
  with TOpenDialog.Create(nil) do
  try
    Filter := OPEN_GOB_FILTER;
    if Execute then
      edtContainer.Text := Filename;
  finally
    Free;
  end;
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.rbtDirClick(Sender: TObject);
begin
  edtDir.Enabled := rbtDir.Checked;
  edtContainer.Enabled := rbtContainer.Checked;
  btnContainer.Enabled := rbtContainer.Checked;
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.ConvertFromDirectory(const sDir: string);
var
  sr: TSearchRec;
  i : integer;
begin
  i := 0;
  if not ForceDirectories(sDir) then
    raise Exception.Create('Unable to create directory');
  if FindFirst(edtDir.Text + '*.mat', faAnyFile, sr) = 0 then begin
    repeat
      inc(i);
      pnlStatus.Caption := Format(' Status: (%d) %s...', [i, sr.Name]);
      pnlStatus.Update;
      if rbtJPEG.Checked then
        MatToJPG(edtDir.Text + sr.Name,
                 edtOutDir.Text + ChangeFileExt(sr.Name, '.jpg'),
                 FaPalette)
      else if rbtTGA.Checked then
        MatToTGA(edtDir.Text + sr.Name,
                 edtOutDir.Text + ChangeFileExt(sr.Name, '.tga'),
                 FaPalette);

    until FindNext(sr) <> 0;
    FindClose(sr);
  end;
  pnlStatus.Caption := Format(' Status: Done, converted %d textures', [i]);
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.ConvertFromContainer(const sFilename: string);
var
  jkc : TJKContainer;
  jkt : TJKTexture;
  i : integer;
  stm : TMemoryStream;
  iCount : integer;
begin
  jkc := TJKContainer.Create;
  stm := TMemoryStream.Create;
  try
    iCount := 0;
    jkc.LoadFromFile(sFilename);
    for i := 0 to jkc.stlFiles.Count - 1 do
      if Pos('mat\', LowerCase(jkc.stlFiles[i])) = 1 then begin
        stm.Clear;
        stm.Position := 0;
        if jkc.ExtractToStream(stm, jkc.stlFiles[i]) then begin
          pnlStatus.Caption := Format(' Status: (%d) %s...', [iCount, jkc.stlFiles[i]]);
          pnlStatus.Update;
          jkt := TJKTexture.Create;
          try
            jkt.Palette := FaPalette;
            jkt.LoadFromStream(stm);
            if rbtJPEG.Checked then
              jkt.SaveAsJPEGToFile(
                edtOutDir.Text + ChangeFileExt(ExtractFileName(jkc.stlFiles[i]), '.jpg'))
            else if rbtTGA.Checked then
              jkt.SaveAsTGAToFile(
                edtOutDir.Text + ChangeFileExt(ExtractFileName(jkc.stlFiles[i]), '.tga'));

            inc(iCount);
          finally
            jkt.Free;
          end;
        end;
      end;
  finally
    stm.Free;
    jkc.Free;
  end;
  pnlStatus.Caption := Format(' Status: Done, converted %d textures', [iCount]);
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.btnDirClick(Sender: TObject);
var
  sDir : string;
begin
  if SelectDirectory('Select Directory', '', sDir) then
    edtDir.Text := sDir;
end;

{------------------------------------------------------------------------------}
procedure TfrmConvTex.btnOutDirClick(Sender: TObject);
var
  sDir : string;
begin
  if SelectDirectory('Select Directory', '', sDir) then
    edtOutDir.Text := sDir;
end;

end.
