unit GTKRadiant_Plugins;

interface

uses
{$IFDEF WIN32}
  Windows, SysUtils;
{$ELSE}
  Wintypes, WinProcs;
{$ENDIF}

const QER_PLUG_VERSION_1                = 1.00;
const QER_PLUG_VERSION                  = 1.70;
const QER_MAX_NAMELEN                   = 1024;

const QER_PLUG_GAME_TEXTURE             = $0001;
const QER_PLUG_GAME_MODEL               = $0002;
const QER_PLUG_GAME_MAP                 = $0004;
const QER_PLUG_GAME_SURFACEDLG          = $0008;
const QER_PLUG_GAME_SURFACEFLAGS        = $0010;  

const MAXPOINTS                         = 16;
const QER_TRANS                         = $00000001;
const QER_NOCARVE                       = $00000002;
const MIN_PATCH_WIDTH	                = 3;
const MIN_PATCH_HEIGHT 	                = 3;
const MAX_PATCH_WIDTH		        = 16;
const MAX_PATCH_HEIGHT	                = 16;

const PATCH_GENERIC                     = $00000000;
const PATCH_CYLINDER                    = $00000001;
const PATCH_BEVEL                       = $00000002;
const PATCH_ENDCAP                      = $00000004;
const PATCH_HEMISPHERE                  = $00000008;
const PATCH_CONE                        = $00000010;
const PATCH_TRIANGLE                    = $00000020;
const PATCH_CAP                         = $00001000;
const PATCH_SEAM                        = $00002000;
const PATCH_THICK                       = $00004000;
const PATCH_BEZIER                      = $00000000;
const PATCH_BSPLINE                     = $10000000;
const PATCH_TYPEMASK                    = $00000fff;
const PATCH_BTYPEMASK                   = $0000f000;
const PATCH_STYLEMASK                   = $ffff0000;

const MAX_FLAGS	                        = 8;

const ECLASS_LIGHT                      = $00000001;
const ECLASS_ANGLE                      = $00000002;
const ECLASS_PATH                       = $00000004;
const ECLASS_MISCMODEL                  = $00000008;
const ECLASS_PLUGINENTITY               = $00000010;

const W_CAMERA	                	= $0001;
const W_XY	        	        = $0002;
const W_XY_OVERLAY                      = $0004;
const W_Z	        	        = $0008;
const W_TEXTURE		                = $0010;
const W_Z_OVERLAY       	        = $0020;
const W_CONSOLE	        	        = $0040;
const W_ENTITY		                = $0080;
const W_CAMERA_IFON                     = $0100;
const W_XZ                              = $0200;
const W_YZ                              = $0400;
const W_GROUP                           = $0800;
const W_MEDIA                           = $1000;
const W_ALL		                = $FFFFFFF;

type
 guint = cardinal;
 gint8 = shortint;
 guint8 = byte;
 gint16 = system.integer;
 guint16 = word;
 guint32 = cardinal;
 PGData = pointer;
 gpointer = pointer;
 Pgchar = ^gchar;
 gchar = char;

 TGtkObject = record
  klass : pointer;                      //PGtkObjectClass
  flags : guint32;
  ref_count : guint;
  object_data : PGData;
 end; {TGtkObject}

 TGtkRequisition = record
  width : gint16;
  height : gint16;
 end; {TGtkRequisition}

 TGtkAllocation = record
  x : gint16;
  y : gint16;
  width : guint16;
  height : guint16;
 end;  {TGtkAllocation}

 PGdkWindow = ^TGdkWindow;
 TGdkWindow = record
  user_data : gpointer;
 end; {TGdkWindow}

 PGdkDrawableWin32Data = ^TGdkDrawableWin32Data;
 TGdkDrawableWin32Data = record
  xid: THandle;
 end; {TGdkDrawableWin32Data}

 TGdkDrawable = record
   user_data:gpointer;
 end; {TGdkDrawable}

 PGdkDrawablePrivate = ^TGdkDrawablePrivate;
 TGdkDrawablePrivate = record
  drawable:TGdkDrawable;
  klass:pointer;                        //GdkDrawableClass
  klass_data:gpointer;
  window_type:guint8;
  ref_count:guint;
  width:guint16;
  height:guint16;
  colormap:pointer;                     //GdkColormap
  destroyed:guint;
 end; {TGdkDrawablePrivate}

 PGtkWidget = ^TGtkWidget;
 TGtkWidget = record
  theobject :TGtkObject;
  private_flags : guint16;
  state : guint8;
  saved_state : guint8;
  name : Pgchar;
  thestyle : pointer;                   //PGtkStyle
  requisition : TGtkRequisition;
  allocation : TGtkAllocation;
  window : PGdkWindow;
  parent : pointer;                     //PGtkWidget
 end; {TGtkWidget}

 TViewType = (YZ, XZ, XY);

 PVec3=^TVec3;
 TVec3=array[0..2] of single;

 PEPair=^TEPair;
 TEPair=record
  next: PEPair;
  key: PChar;
  value: PChar;
 end; {TEPair}

 PPatchmesh=^TPatchMesh;
 TPatchMesh=record
  {todo}
 end; {TPatchMesh}

 TBrushPrimit_TexDef = record
  Coords:array[0..1] of array[0..2] of TVec3;
 end; {TBrushPrimit_TexDef}

 PQERPointData=^TQERPointData;
 TQERPointData=record
  Count:integer;
  Vectors:PVec3;
 end; {TQERPointData}

 PQERTextureInfo=^TQERTextureInfo;
 TQERTextureInfo=record
  TextureExtension:array[0..QER_MAX_NAMELEN-1] of Char;
  HiColor:boolean;
  WadStyle:boolean;
  HalfLife:boolean;
 end; {TQERTextureInfo}

 PQERModelInfo=^TQERModelInfo;
 TQERModelInfo=record
  ModelExtension:array[0..QER_MAX_NAMELEN-1] of Char;
  Skinned:boolean;
  Multipart:boolean;
 end; {TQERModelInfo}

 PQERFaceData=^TQERFaceData;
 TQERFaceData=record
  TextureName:array[0..(QER_MAX_NAMELEN-1)] of char;
  Contents:longint;
  Flags:longint;
  Value:longint;
  Shift:array[0..1] of single;
  Rotate:single;
  Scale:array[0..1] of single;
  Vec1,Vec2,Vec3:TVec3;
  BPrimit:boolean;
  brushprimit_texdef:TBrushPrimit_TexDef;
 end; {TQERFaceData}

TQERFuncTable=record
  Size:integer;
  pCreateBrush:procedure(vMin,vMax:TVec3);pascal;
  pCreateBrushHandle:function:pointer;stdcall;
  pDeleteBrushHandle:procedure(pBrush:pointer);stdcall;
  pCommitBrushHandleToMap:procedure(pBrush:pointer);stdcall;
  pAddFace:procedure(pBrush:pointer;v1,v2,v3:TVec3);stdcall;
  pAddFaceData:procedure(pBrush:pointer;pFaceData:pQERFaceData);stdcall;
  pGetFaceData:function(pBrush:pointer;FaceIndex:integer):PQERFaceData;stdcall;
  pGetFaceCount:function(pBrush:pointer):integer;stdcall;
  pSetFaceData:procedure(pBrush:pointer;FaceIndex:integer;pFaceData:PQERFaceData);stdcall;
  pDeleteFace:procedure(pBrush:pointer;FaceIndex:integer);stdcall;
  pTextureBrush:procedure(pBrush:pointer;pName:LPCSTR);stdcall;
  pBuildBrush:procedure(pBrush:pointer);stdcall;
  pSelectBrush:procedure(pBrush:pointer);stdcall;
  pDeselectBrush:procedure(pBrush:pointer);stdcall;
  pDeselectAllBrushes:procedure;stdcall;
  pDeleteSelection:procedure;stdcall;
  pGetPoints:procedure(nMax:integer;PointData:TQERPointData;pMsg:LPCSTR);stdcall;
  pPositionView:procedure(v1,v2,v3:TVec3);stdcall;
  pSelectedBrushCount:function:integer;stdcall;
  pAllocateSelectedBrushHandles:function:integer;stdcall;
  pReleaseSelectedBrushHandles:procedure;stdcall;
  pGetSelectedBrushHandle:function(BrushIndex:integer):pointer;stdcall;
  pActiveBrushCount:function:integer;stdcall;
  pAllocateActiveBrushHandles:function:integer;stdcall;
  pReleaseActiveBrushHandles:procedure;stdcall;
  pGetActiveBrushHandle:function(BrushIndex:integer):pointer;stdcall;
  pTextureCount:function:integer;stdcall;
  pGetTexture:function(TexIndex:integer):pointer;stdcall;
  pGetCurrentTexture:function:LPCSTR;stdcall;
  pSetCurrentTexture:procedure(pname:LPCSTR);stdcall;
  pGetEClassCount:function:integer;stdcall;
  pGetEClass:function(ClassIndex:integer):LPCSTR;stdcall;
  pResetPlugins:procedure;stdcall;
  pLoadTextureRGBA:procedure(pTex:pointer);stdcall;
  pGetEntityCount:function:integer;stdcall;
  pGetEntityHandle:function(EntityIndex:integer):pointer;stdcall;
  pGetEntityKeyValList:function(pEntity:pointer):TEPair;stdcall;
  pSetEntityKeyValList:procedure(pEntity:pointer;entityData:PEPair);stdcall;
  pAllocateEntityBrushHandles:function(pEntity:pointer):integer;stdcall;
  pReleaseEntityBrushHandles:procedure;stdcall;
  pGetEntityBrushHandle:function(EntityIndex:integer):pointer;stdcall;
  pCreateEntityHandle:function:pointer;stdcall;
  pCommitBrushHandleToEntity:procedure(pBrush,pEntity:pointer);stdcall;
  pCommitEntityHandleToMap:procedure(pEntity:pointer);stdcall;
  pAllocateEpair:function(e1,e2:pChar):PEPair;stdcall;
  pSetScreenUpdate:procedure(ScreenUpdate:integer);stdcall;
  pBuildBrush2:procedure(pBrush:pointer;convert:integer);stdcall;
  pRequestInterface:function(a:TGUID;b:pointer):integer;stdcall;
  pError:procedure(pMsg:LPCSTR);stdcall;
  pLoadFile:function(pLocation:PChar;buffer:pointer):integer;stdcall;
  pExpandReletivePath:function(pName:PChar):PChar;stdcall;
  pConvertDOSToUnixName:procedure(pDst:PChar;const pSrc:PChar);stdcall;
  pHasShader:function(const pName:PChar):integer;stdcall;
  pTextureLoadSkin:function(pName:PChar;Width,Height:integer):integer;stdcall;
  pRadiantFree:function(p:pointer):pointer;stdcall;
  pGetGamePath:function:pChar;stdcall;
  pGetQERPath:function:pChar;stdcall;
  pAllocateActivePatchHandles:function:integer;stdcall;
  pAllocateSelectedPatchHandles:function:integer;stdcall;
  pReleasePatchHandles:procedure;stdcall;
  pGetPatchData:function(PatchIndex:integer):PPatchMesh;stdcall;
  pDeletePatch:procedure(PatchIndex:integer);stdcall;
  pCreatePatchHandle:function:integer;stdcall;
  pCommitPatchHandleToMap:procedure(a:integer;pPatch:PPatchMesh;TexName:char);stdcall;
//fix these
  pLoadImage:pointer;
  pMessageBox:pointer;
  pFileDialog:pointer;
  pDirDialog:pointer;
  pColorDialog:pointer;
  pLoadBitmap:pointer;
//
  pProfileGetDirectory:function:PChar;
  pProfileSaveInt:function(pFileName,pSection,pKey:PChar;Value:integer):boolean;
  pProfileSaveString:function(pFileName,pSection,pKey:PChar;const Value:PChar):boolean;
  pProfileLoadInt:function(pFileName,pSection,pKey:PChar;DefaultValue:integer):integer;
  pProfileLoadString:function(pFileName,pSection,pKey:PChar;const DefaultValue:PChar):PChar;
  pSysUpdateWindows:procedure(Bits:integer);
  pSysBeep:procedure;stdcall;
  pSysPrintf:procedure(pMsg:PChar);
  pSysFPrintf:procedure(Flag:integer;const pMsg:PChar);
  pSysBeginWait:procedure;stdcall;
  pSysEndWait:procedure;stdcall;
  pSysSetTitle:procedure(pTitle:PChar);
  pSysStatus:procedure(pStatus:PChar;Part:integer);
  pMapNew:procedure;stdcall;
  pMapFree:procedure;stdcall;
  pMapBuildBrushData:procedure;stdcall;
  pMapIsBrushFiltered:function(pBrush:pointer):boolean;
  pMapStartPosition:procedure;stdcall;
  pMapRegionOff:procedure;stdcall;
  pSetBuildWindingsNoTexBuild:procedure(a:boolean);
  pSaveAsDialog:procedure(bRegion:boolean);
  pPointFileClear:procedure;stdcall;
  pCSGMakeHollow:procedure;stdcall;
  pRegionSpawnPoint:pointer;                            //???
 end; {TQERFuncTable}

TQERPlug=class
  FuncTable:TQERFuncTable;
  constructor Create;
  procedure CreateBrush(vMin,vMax:TVec3);
  function CreateBrushHandle:pointer;
  procedure DeleteBrushHandle(pBrush:pointer);
  procedure CommitBrushHandleToMap(pBrush:pointer);
  procedure AddFace(pBrush:pointer;v1,v2,v3:TVec3);
  procedure AddFaceData(pBrush:pointer;pFaceData:pQERFaceData);
  function GetFaceData(pBrush:pointer;FaceIndex:integer):PQERFaceData;
  function GetFaceCount(pBrush:pointer):integer;
  procedure SetFaceData(pBrush:pointer;FaceIndex:integer;pFaceData:PQERFaceData);
  procedure DeleteFace(pBrush:pointer;faceIndex:integer);
  procedure TextureBrush(pBrush:pointer;pName:LPCSTR);
  procedure BuildBrush(pBrush:pointer);
  procedure SelectBrush(pBrush:pointer);
  procedure DeselectBrush(pBrush:pointer);
  procedure DeselectAllBrushes;
  procedure DeleteSelection;
  procedure GetPoints(nMax:integer;pointData:TQERPointData;pMsg:LPCSTR);
  procedure PositionView(v1,v2,v3:TVec3);
  function SelectedBrushCount:integer;
  function AllocateSelectedBrushHandles:integer;
  procedure ReleaseSelectedBrushHandles;
  function GetSelectedBrushHandle(brushIndex:integer):pointer;
  function ActiveBrushCount:integer;
  function AllocateActiveBrushHandles:integer;
  procedure ReleaseActiveBrushHandles;
  function GetActiveBrushHandle(brushIndex:integer):pointer;
  function TextureCount:integer;
  function GetTexture(TexIndex:integer):pointer;
  function GetCurrentTexture:LPCSTR;
  procedure SetCurrentTexture(pname:LPCSTR);
  function GetEClassCount:integer;
  function GetEClass(ClassIndex:integer):LPCSTR;
  procedure ResetPlugins;
  procedure LoadTextureRGBA(pTex:pointer);
  function GetEntityCount:integer;
  function GetEntityHandle(EntityIndex:integer):pointer;
  function GetEntityKeyValList(pEntity:pointer):TEPair;
  procedure SetEntityKeyValList(pEntity:pointer;EntityData:PEPair);
  function AllocateEntityBrushHandles(pEntity:pointer):integer;
  procedure ReleaseEntityBrushHandles;
  function GetEntityBrushHandle(EntityIndex:integer):pointer;
  function CreateEntityHandle:pointer;
  procedure CommitBrushHandleToEntity(pBrush,pEntity:pointer);
  procedure CommitEntityHandleToMap(pEntity:pointer);
  function AllocateEpair(e1,e2:pChar):PEPair;
  procedure SetScreenUpdate(ScreenUpdate:integer);
  procedure BuildBrush2(pBrush:pointer;Convert:integer);
  function RequestInterface(a:TGUID;b:pointer):integer;
  procedure Error(s:string);
  function LoadFile(pLocation:PChar;pBuffer:pointer):integer;
  function ExpandReletivePath(pName:PChar):PChar;
  procedure ConvertDOSToUnixName(pDst:PChar;const pSrc:PChar);
  function HasShader(pName:PChar):integer;
  function TextureLoadSkin(pName:PChar;Width,Height:integer):integer;
  function RadiantFree(p:pointer):pointer;
  function GetGamePath:PChar;
  function GetQERPath:PChar;
  function AllocateActivePatchHandles:integer;
  function AllocateSelectedPatchHandles:integer;
  procedure ReleasePatchHandles;
  function GetPatchData(PatchIndex:integer):PPatchMesh;
  procedure DeletePatch(PatchIndex:integer);
  function CreatePatchHandle:integer;
  procedure CommitPatchHandleToMap(a:integer;pPatch:PPatchMesh;TexName:char);
//add stuff here
  procedure SysUpdateWindows(Bits:integer);
 end; {TQERPlug}

implementation

constructor TQERPlug.Create;
begin
 FuncTable.Size:=SizeOf(TQERFuncTable);
end;

procedure TQERPlug.CreateBrush(vMin,vMax:TVec3);
begin
 FuncTable.pCreateBrush(vMax,vMin);
end;

function TQERPlug.CreateBrushHandle:pointer;
begin
 Result:=FuncTable.pCreateBrushHandle;
end;

procedure TQERPlug.DeleteBrushHandle(pBrush:pointer);
begin
 FuncTable.pDeleteBrushHandle(pBrush);
end;

procedure TQERPlug.CommitBrushHandleToMap(pBrush:pointer);
begin
 FuncTable.pCommitBrushHandleToMap(pBrush);
end;

procedure TQERPlug.AddFace(pBrush:pointer;v1,v2,v3:TVec3);
begin
 FuncTable.pAddFace(pBrush,v1,v2,v3);
end;

procedure TQERPlug.AddFaceData(pBrush:pointer;pFaceData:pQERFaceData);
begin
 FuncTable.pAddFaceData(pBrush,pFaceData);
end;

function TQERPlug.GetFaceData(pBrush:pointer;FaceIndex:integer):PQERFaceData;
begin
 Result:=FuncTable.pGetFaceData(pBrush,FaceIndex);
end;

function TQERPlug.GetFaceCount(pBrush:pointer):integer;
begin
 Result:=FuncTable.pGetFaceCount(pBrush);
end;

procedure TQERPlug.SetFaceData(pBrush:pointer;FaceIndex:integer;pFaceData:PQERFaceData);
begin
 FuncTable.pSetFaceData(pBrush,FaceIndex,pFaceData);
end;

procedure TQERPlug.DeleteFace(pBrush:pointer;FaceIndex:integer);
begin
 FuncTable.pDeleteFace(pBrush,FaceIndex);
end;


procedure TQERPlug.TextureBrush(pBrush:pointer;pName:LPCSTR);
begin
 FuncTable.pTextureBrush(pBrush,pName);
end;

procedure TQERPlug.BuildBrush(pBrush:pointer);
begin
 FuncTable.pBuildBrush(pBrush);
end;

procedure TQERPlug.SelectBrush(pBrush:pointer);
begin
 FuncTable.pSelectBrush(pBrush);
end;

procedure TQERPlug.DeselectBrush(pBrush:pointer);
begin
 FuncTable.pDeselectBrush(pBrush);
end;

procedure TQERPlug.DeselectAllBrushes;
begin
 FuncTable.pDeselectAllBrushes;
end;

procedure TQERPlug.DeleteSelection;
begin
 FuncTable.pDeleteSelection;
end;

procedure TQERPlug.GetPoints(nMax:integer;PointData:TQERPointData;pMsg:LPCSTR);
begin
 FuncTable.pGetPoints(nMax,PointData,pMsg);
end;

procedure TQERPlug.PositionView(v1,v2,v3:TVec3);
begin
 FuncTable.pPositionView(v1,v2,v3);
end;

function TQERPlug.SelectedBrushCount:integer;
begin
 Result:=FuncTable.pSelectedBrushCount;
end;

function TQERPlug.AllocateSelectedBrushHandles:integer;
begin
 Result:=FuncTable.pAllocateSelectedBrushHandles;
end;

procedure TQERPlug.ReleaseSelectedBrushHandles;
begin
 FuncTable.pReleaseSelectedBrushHandles;
end;

function TQERPlug.GetSelectedBrushHandle(BrushIndex:integer):pointer;
begin
 Result:=FuncTable.pGetSelectedBrushHandle(BrushIndex)
end;

function TQERPlug.ActiveBrushCount:integer;
begin
 Result:=FuncTable.pActiveBrushCount;
end;

function TQERPlug.AllocateActiveBrushHandles:integer;
begin
 Result:=FuncTable.pAllocateActiveBrushHandles;
end;

procedure TQERPlug.ReleaseActiveBrushHandles;
begin
 FuncTable.pReleaseActiveBrushHandles;
end;

function TQERPlug.GetActiveBrushHandle(BrushIndex:integer):pointer;
begin
 Result:=FuncTable.pGetActiveBrushHandle(BrushIndex)
end;

function TQERPlug.TextureCount:integer;
begin
 Result:=FuncTable.pTextureCount;
end;

function TQERPlug.GetTexture(TexIndex:integer):pointer;
begin
 Result:=FuncTable.pGetTexture(TexIndex);
end;

function TQERPlug.GetCurrentTexture:LPCSTR;
begin
 Result:=FuncTable.pGetCurrentTexture;
end;

procedure TQERPlug.SetCurrentTexture(pName:LPCSTR);
begin
 FuncTable.pSetCurrentTexture(pName);
end;

function TQERPlug.GetEClassCount:integer;
begin
 Result:=FuncTable.pGetEClassCount;
end;

function TQERPlug.GetEClass(ClassIndex:integer):LPCSTR;
begin
 Result:=FuncTable.pGetEClass(ClassIndex);
end;

procedure TQERPlug.ResetPlugins;
begin
 FuncTable.pResetPlugins;
end;

procedure TQERPlug.LoadTextureRGBA(pTex:pointer);
begin
 FuncTable.pLoadTextureRGBA(pTex);
end;

function TQERPlug.GetEntityCount:integer;
begin
 Result:=FuncTable.pGetEntityCount;
end;

function TQERPlug.GetEntityHandle(EntityIndex:integer):pointer;
begin
 Result:=FuncTable.pGetEntityHandle(EntityIndex);
end;

function TQERPlug.GetEntityKeyValList(pEntity:pointer):TEPair;
begin
 Result:=FuncTable.pGetEntityKeyValList(pEntity);
end;

procedure TQERPlug.SetEntityKeyValList(pEntity:pointer;EntityData:PEPair);
begin
 FuncTable.pSetEntityKeyValList(pEntity,EntityData);
end;

function TQERPlug.AllocateEntityBrushHandles(pEntity:pointer):integer;
begin
 Result:=FuncTable.pAllocateEntityBrushHandles(pEntity);
end;

procedure TQERPlug.ReleaseEntityBrushHandles;
begin
 FuncTable.pReleaseEntityBrushHandles;
end;

function TQERPlug.GetEntityBrushHandle(EntityIndex:integer):pointer;
begin
 Result:=FuncTable.pGetEntityBrushHandle(EntityIndex);
end;

function TQERPlug.CreateEntityHandle:pointer;
begin
 Result:=FuncTable.pCreateEntityHandle;
end;

procedure TQERPlug.CommitBrushHandleToEntity(pBrush,pEntity:pointer);
begin
 FuncTable.pCommitBrushHandleToEntity(pBrush,pEntity);
end;

procedure TQERPlug.CommitEntityHandleToMap(pEntity:pointer);
begin
 FuncTable.pCommitEntityHandleToMap(pEntity);
end;

function TQERPlug.AllocateEpair(e1,e2:pChar):PEPAir;
begin
 Result:=FuncTable.pAllocateEpair(e1,e2);
end;

procedure TQERPlug.SetScreenUpdate(ScreenUpdate:integer);
begin
 FuncTable.pSetScreenUpdate(ScreenUpdate);
end;

procedure TQERPlug.BuildBrush2(pBrush:pointer;Convert:integer);
begin
 FuncTable.pBuildBrush2(pBrush,Convert);
end;

function TQERPlug.RequestInterface(a:TGUID;b:pointer):integer;
begin
 Result:=FuncTable.pRequestInterface(a,b);
end;

procedure TQERPlug.Error(s:string);
begin
 FuncTable.pError(PChar(s+#10));
end;

function TQERPlug.LoadFile(pLocation:PChar;pBuffer:pointer):integer;
begin
 Result:=FuncTable.pLoadFile(pLocation,pBuffer);
end;

function TQERPlug.ExpandReletivePath(pName:PChar):PChar;
begin
 Result:=FuncTable.pExpandReletivePath(pName);
end;

procedure TQERPlug.ConvertDOSToUnixName(pDst:PChar;const pSrc:PChar);
begin
 FuncTable.pConvertDOSToUnixName(pDst,pSrc);
end;

function TQERPlug.HasShader(pName:PChar):integer;
begin
 Result:=FuncTable.pHasShader(pName);
end;

function TQERPlug.TextureLoadSkin(pName:PChar;Width,Height:integer):integer;
begin
 Result:=FuncTable.pTextureLoadSkin(pName,Width,Height)
end;

function TQERPlug.RadiantFree(p:pointer):pointer;
begin
 Result:=FuncTable.pRadiantFree(p);
end;

function TQERPlug.GetGamePath:PChar;
begin
 Result:=FuncTable.pGetGamePath;
end;

function TQERPlug.GetQERPath:PChar;
begin
 Result:=FuncTable.pGetQERPath;
end;

function TQERPlug.AllocateActivePatchHandles:integer;
begin
 Result:=FuncTable.pAllocateActivePatchHandles;
end;

function TQERPlug.AllocateSelectedPatchHandles:integer;
begin
 Result:=FuncTable.pAllocateSelectedPatchHandles;
end;

procedure TQERPlug.ReleasePatchHandles;
begin
 FuncTable.pReleasePatchHandles;
end;

function TQERPlug.GetPatchData(PatchIndex:integer):PPatchMesh;
begin
 Result:=FuncTable.pGetPatchData(PatchIndex);
end;

procedure TQERPlug.DeletePatch(PatchIndex:integer);
begin
 FuncTable.pDeletePatch(PatchIndex);
end;

function TQERPlug.CreatePatchHandle:integer;
begin
 Result:=FuncTable.pCreatePatchHandle;
end;

procedure TQERPlug.CommitPatchHandleToMap(a:integer;pPatch:PPatchMesh;TexName:char);
begin
 FuncTable.pCommitPatchHandleToMap(a,pPatch,TexName);
end;

//add stuff here

procedure TQERPlug.SysUpdateWindows(Bits:integer);
begin
 FuncTable.pSysUpdateWindows(Bits);
end;


end.
