(*
 * Copyright (c) 2002 Andr di Geronimo (adg@home.se)
 *)
 
unit O_brush;

interface

uses
  Windows, Classes, SysUtils,
  P_str, O_geo, O_ComEnt;

type
  TBrush = T3DMesh;

  TBrushLevel = class(TObject)
  private
    FlstWorldBrushes : T3DMeshList;
    FlstWorldThings : TStringList;
    FlstWorldLights : TList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function SaveToFile(const sFilename : string): boolean;
    function SaveToStl(stl : TStringList): boolean;
    property lstWorldBrushes : T3DMeshList read FlstWorldBrushes;
    property lstWorldThings : TStringList read FlstWorldThings;
    property lstWorldLights : TList read FlstWorldLights;
  end;

implementation

{------------------------------------------------------------------------------}
constructor TBrushLevel.Create;
begin
  FlstWorldBrushes  := T3DMeshList.Create;
  FlstWorldThings := TStringList.Create;
  FlstWorldLights := TList.Create;
end;

{------------------------------------------------------------------------------}
destructor TBrushLevel.Destroy;
begin
  Clear;
  FlstWorldBrushes.Free;
  inherited;
end;

{------------------------------------------------------------------------------}
procedure TBrushLevel.Clear;
var
  i : integer;
  pThg : PThing;
  pLit : PLight;
begin
  FlstWorldBrushes.FreeAll;
  FlstWorldBrushes.Clear;

  for i := 0 to FlstWorldThings.Count - 1 do begin
    pThg := PThing(FlstWorldThings.Objects[i]);
    Dispose(pThg);
  end;
  FlstWorldThings.Clear;

  for i := 0 to FlstWorldLights.Count - 1 do begin
    pLit := PLight(FlstWorldLights[i]);
    Dispose(pLit);
  end;
  FlstWorldLights.Clear;
end;

{------------------------------------------------------------------------------}
function TBrushLevel.SaveToFile(const sFilename: string): boolean;
var
  stl : TStringList;
begin
  stl := TStringList.Create;
  try
    Result := SaveToStl(stl);
    if Result then
      stl.SaveToFile(sFilename);
  finally
    stl.Free;
  end;
end;

{------------------------------------------------------------------------------}
function TBrushLevel.SaveToStl(stl: TStringList): boolean;
const
  INDENT  = '';
  INDENT2 = INDENT + INDENT;

  {----------------------------------------------}
  function FmtVertex(vx : TVertex) : string;
  begin
    Result :=
      Format('( %.5f %.5f %.5f )', [vx.v.x, vx.v.y, vx.v.z]);
  end;

var
  i, j : integer;
  bsh : TBrush;
  sur : TSurface;
  sSurfaceLine : string;
begin
  stl.Add('{');
  stl.Add(INDENT + '"classname" "worldspawn"');

  for i := 0 to FlstWorldBrushes.Count - 1 do begin
    stl.Add(Format(INDENT + '// Brush %d', [i]));
    stl.Add(INDENT + '{');
    bsh := FlstWorldBrushes[i];

    for j := 0 to bsh.Surfaces.Count - 1 do begin
      sur := bsh.Surfaces[j];
      if sur.Vertices.Count >= 3 then begin
        sSurfaceLine := INDENT2 + 
          FmtVertex(sur.Vertices[0]) + ' ' +
          FmtVertex(sur.Vertices[1]) + ' ' +
          FmtVertex(sur.Vertices[2]) + ' ' +
          {sur.sMaterial} 'unnamed' +
          ' 0 0 0 0.500000 0.500000 0 0 0';
        stl.Add(sSurfaceLine);
      end;
    end;
    stl.Add(INDENT + '}');
  end;
  stl.Add('}');
  Result := TRUE;
end;

end.
