(*
 * Copyright (c) 2002 Andr di Geronimo (adg@home.se)
 *)
 
unit O_jkc;

interface

uses
  Windows, Classes, SysUtils;

type
  TGOBHeader = record
    tag : array[0..2] of char;   {'GOB '}
    version : byte;              {Apparently - version= x20 }
    Offset1 : Longint;           {Offset to first file size from begining of file= x14 }
    Offset2 : Longint;           {Offset to #of GobItems from file beginning= x0C }
    Count : Longint;             {# of items in gob file }
  end;

  PGobItem = ^TGobItem;
  TGobItem = record
    Offset : Longint;                 {offset from begining of file}
    Size : Longint;                   {Length of file}
    Filename : array[0..127] of char; {path and name of file}
  end;

  TJKContainer = class(TObject)
  private
    FsFilename : string;
    FstlFiles : TStringList;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Clear;
    function LoadFromStream(stm : TStream) : boolean;
    function LoadFromFile(const sFilename : string) : boolean;
    function ExtractToStream(stm : TStream ; sItemFilename : string) : boolean;
    function ExtractToStl(stl : TStrings ; sItemFilename : string) : boolean;
    property stlFiles : TStringLIst read FstlFiles;
    property FileName : string read FsFilename write FsFilename;
  end;

implementation

{------------------------------------------------------------------------------}
constructor TJKContainer.Create;
begin
  FstlFiles := TStringList.Create;
end;

{------------------------------------------------------------------------------}
destructor TJKContainer.Destroy;
begin
  Clear;
  FstlFiles.Free;
  inherited;
end;

{------------------------------------------------------------------------------}
procedure TJKContainer.Clear;
var
  i : integer;
begin
  for i := 0 to FstlFiles.Count - 1 do
    Dispose(PGobItem(FstlFiles.Objects[i]));
  FstlFiles.Clear;
end;

{------------------------------------------------------------------------------}
function TJKContainer.LoadFromStream(stm: TStream): boolean;
var
  recHead : TGOBHeader;
  pItem : PGobItem;
  i : integer;
begin
  Clear;
  stm.ReadBuffer(recHead, SizeOf(recHead));
  for i := 0 to recHead.Count - 1 do begin
    New(pItem);
    stm.ReadBuffer(pItem^, SizeOf(TGobItem));
    FstlFiles.AddObject(LowerCase(TrimRight(pItem^.Filename)), TObject(pItem))
  end;
  FstlFiles.Sort;
  Result := TRUE;
end;

{------------------------------------------------------------------------------}
function TJKContainer.LoadFromFile(const sFilename : string): boolean;
var
  stm : TFileStream;
begin
  if FileExists(sFilename) then begin
    FsFilename := sFilename;
    stm := TFileStream.Create(sFilename, fmOpenRead);
    try
      Result := Self.LoadFromStream(stm);
    finally
      stm.Free;
    end;
  end
  else
    Result := TRUE;
end;

{------------------------------------------------------------------------------}
function TJKContainer.ExtractToStream(stm: TStream ; sItemFilename : string): boolean;
var
  i : integer;
  stmGob : TFileStream;
  pItem : PGobItem;
  pBuff : pointer;
begin
  i := FstlFiles.IndexOf(sItemFilename);
  if i <> -1 then begin
    stmGob := TFileStream.Create(FsFilename, fmOpenRead);
    pItem := PGobItem(FstlFiles.Objects[i]);

    pBuff := AllocMem(pItem^.Size);
    try
      stmGob.Seek(pItem^.Offset, soFromBeginning);
      stmGob.ReadBuffer(pBuff^, pItem^.Size);
      stm.WriteBuffer(pBuff^, pItem^.Size);
      Result := TRUE;
    finally
      stmGob.Free;
      FreeMem(pBuff);
    end;
    
  end
  else
    Result := FALSE;
end;

{------------------------------------------------------------------------------}
function TJKContainer.ExtractToStl(stl: TStrings;
  sItemFilename: string): boolean;
var
  stm : TStringStream;
begin
  stm := TStringStream.Create('');
  try
    Result := Self.ExtractToStream(stm, sItemFilename);
    if Result then
      stl.Text := stm.DataString;
  finally
    stm.Free;
  end;
end;

end.
 