(*
 * Copyright (c) 2002 Andr di Geronimo (adg@home.se)
 *)
 
unit P_vec;

interface

type
  TVector = record x, y, z : single; end;
  TUV = record u, v : single; end;
  TCamera = record
    pos : TVector;
    rot : TVector;
  end;

function vec(x, y, z : single) : TVector;
function vecValid(v : TVector) : boolean;
function vecLength(vec : TVector) : single;
function vecAdd(v1, v2 : TVector) : TVector;
function vecSubtract(v1, v2 : TVector) : TVector;
function vecMultiply(v1, v2 : TVector) : TVector;
function vecNormalize(vec : TVector) : TVector;
function vecDotProduct(vec1, vec2 : TVector) : single;
function vecCrossProduct(vec1, vec2 : TVector) : TVector;
function CalcNormal(v1, v2, v3 : TVector) : TVector;

implementation

{------------------------------------------------------------------------------}
function vec(x, y, z : single) : TVector;
begin
  Result.x := x;
  Result.y := y;
  Result.z := z;
end;

{------------------------------------------------------------------------------}
function vecValid(v : TVector) : boolean;
begin
  Result := not ((v.x = 0) and (v.y = 0) and (v.z = 0));
end;

{------------------------------------------------------------------------------}
function vecLength(vec : TVector) : single;
begin
  Result := sqrt((vec.x * vec.x) +
                 (vec.y * vec.y) +
                 (vec.z * vec.z));
end;

{------------------------------------------------------------------------------}
function vecAdd(v1, v2 : TVector) : TVector;
begin
  Result.x := v1.x + v2.x;
  Result.y := v1.y + v2.y;
  Result.z := v1.z + v2.z;
end;

{------------------------------------------------------------------------------}
function vecSubtract(v1, v2 : TVector) : TVector;
begin
  Result.x := v1.x - v2.x;
  Result.y := v1.y - v2.y;
  Result.z := v1.z - v2.z;
end;

{------------------------------------------------------------------------------}
function vecMultiply(v1, v2 : TVector) : TVector;
begin
  Result.x := v1.x * v2.x;
  Result.y := v1.y * v2.y;
  Result.z := v1.z * v2.z;
end;

{------------------------------------------------------------------------------}
function vecNormalize(vec : TVector) : TVector;
var
  len : single;
begin
  len := vecLength(vec);
  if len = 0 then
    len := 1;
  Result.x := vec.x / len;
  Result.y := vec.y / len;
  Result.z := vec.z / len;
end;

{------------------------------------------------------------------------------}
function vecDotProduct(vec1, vec2 : TVector) : single;
begin
  Result := vec1.x * vec2.x +
            vec1.y * vec2.y +
            vec1.z * vec2.z;
end;

{------------------------------------------------------------------------------}
function vecCrossProduct(vec1, vec2 : TVector) : TVector;
begin
  Result.x := vec1.y * vec2.z - vec1.z * vec2.y;
  Result.y := vec1.z * vec2.x - vec1.x * vec2.z;
  Result.z := vec1.x * vec2.y - vec1.y * vec2.x;
end;

{------------------------------------------------------------------------------}
function CalcNormal(v1, v2, v3 : TVector) : TVector;
var
  vec1, vec2 : TVector;
begin
  vec1 := vecSubtract(v1, v2);
  vec2 := vecSubtract(v2, v3);
  
  Result := vecCrossProduct(vec1, vec2);
  Result := vecNormalize(Result);
end;

end.
