(*
 * Copyright (c) 2002 Andr di Geronimo (adg@home.se)
 *)
 
unit main;

interface

//TODO:Convert textures coords (UVs -> scale/rot/pos)
//TODO:Kill corner Z fighting
//TODO:Skip surfaces marked as sky (Option?)
//TODO:Things: "JK to JK2" conversion table 
//TODO:Create a PK3 with converted textures?
//TODO:Support for MAT 16bit and 1bit(Single Color)

// 64/8 = 8 feet = 2.5 metres approx

uses
  SysUtils, Windows, Classes, Dialogs, Registry,
  {$IFDEF Q3RADIANT}QERadiant_Plugins,{$ENDIF}
  {$IFDEF GTKRADIANT}GTKRadiant_Plugins,{$ENDIF}
  O_jkl, O_jkc, O_jkt, O_brush, O_geo, P_vec, O_ComEnt,
  FGobDlg, FConvTex, FStatus;

const
{$IFDEF GTKRADIANT}
  PLUGIN_NAME = 'JKL Import for GtkRadiant';
{$ENDIF}
{$IFDEF Q3RADIANT}
  PLUGIN_NAME = 'JKL Import for JK2Radiant';
{$ENDIF}

  PLUGIN_MENU_NAME = 'JKL Import';
  PLUGIN_VERSION   = '1.2';
  CMD_ABOUT        = 'About...';
  CMD_CONFIGURE    = 'Configure...';
  CMD_IMPORT       = 'Import Level...';
  CMD_CONVTEX      = 'Batch Convert Textures...';
  PLUGIN_COMMANDS  = CMD_ABOUT + ';-;'+
                     CMD_CONFIGURE + ';'+
                     CMD_CONVTEX + ';'+
                     CMD_IMPORT + ';';

  INI_NONVISTEX       = 'NonVisTex';
  INI_RESOURCEGOBS    = 'ResourceGobs';
  INI_OUTPUTTEXFORMAT = 'OutputTexFormat';
  INI_JKPALETTE       = 'JKPalette';
  INI_SCALE           = 'Scale';
  INI_BRUSHDEPTH      = 'BrushDepth';

  DEF_SCALE = 64 + 64 + 64 + 64;

function QERPlug_GetName : LPCSTR; stdcall;
function QERPlug_GetCommandList : LPCSTR; stdcall;
function QERPlug_GetFuncTable : pointer; stdcall;
procedure QERPlug_Dispatch(p:LPCSTR; vMin, vMax : TVec3 ;
  bSingleBrush : boolean) ; stdcall;

procedure QERSysMsg(const sMsg : string);

function DoImport : boolean;
function DoConvTex : boolean;
function JklToBsl(jkl : TJKLevel ; bsl : TBrushLevel ;
  procStatus : TCallbackStatus = nil) : boolean;
function SecToBrush(jkl : TJKLevel ; bsl : TBrushLevel) : boolean;
function ExportLevelTextures(jkl : TJKLevel ;
  const sJKLFile, sGobFile : string ;
  procStatus : TCallbackStatus = nil) : integer;
function ExportBslToRadiant(bsl : TBrushLevel ;
  const sTexPath : string ; procStatus : TCallbackStatus = nil) : boolean;
function Scale : double;
function BrushDepth : double;
function GetGamePath : string;

var
  FuncTable : TQERFuncTable;
  stlSettings : TStringList;

implementation

uses
  ShellApi, Forms, Controls;

{------------------------------------------------------------------------------}
function QERPlug_GetName : LPCSTR; stdcall;
begin
  Result := PLUGIN_MENU_NAME;
end;

{------------------------------------------------------------------------------}
function QERPlug_GetCommandList : LPCSTR; stdcall;
begin
  Result := PLUGIN_COMMANDS;
end;

{------------------------------------------------------------------------------}
function QERPlug_GetFuncTable : pointer; stdcall;
begin
  FuncTable.Size := SizeOf(FuncTable);
{$IFDEF Q3RADIANT}
  FuncTable.Version := QER_PLUG_VERSION;
{$ENDIF}
  Result := @FuncTable;
end;

{------------------------------------------------------------------------------}
procedure QERPlug_Dispatch(p:LPCSTR; vMin, vMax : TVec3 ;
  bSingleBrush:boolean) ; stdcall;
var
  sIniFile : string;
  szModuleFilename : PChar;
begin
  szModuleFilename := AllocMem(MAX_PATH);
  try
    GetModuleFileName(HInstance, szModuleFilename, MAX_PATH);
    sIniFile := ExtractFilePath(szModuleFilename) + 'jklimport.ini';//FuncTable.pGetQERPath + 'plugins\JKLImport.ini';
  finally
    FreeMem(szModuleFilename);
  end;

  stlSettings := TStringList.Create;
  try

    if FileExists(sIniFile) then
      stlSettings.LoadFromFile(sIniFile);

    try
      if p = CMD_IMPORT then
        DoImport
      else if p = CMD_CONVTEX then
        DoConvTex
      else if p = CMD_CONFIGURE then
        //TODO: Configure dialog
        ShellExecute(0, 'open', PChar(sIniFile), nil, nil, SW_SHOW)
      else if p = CMD_ABOUT then
        MessageBox(0, PChar('JKL Import v'+PLUGIN_VERSION+' for ' + PLUGIN_NAME + #10#10 +
                            'Written by '#10 +
                            '  Andr di Geroniomo (adg@home.se)'#10#10 +
                            'Thanks to ' + #10 +
                            '  Craig Urquhart'), PChar(''), 0);
    except
      on e : exception do begin
        QERSysMsg('Failed: ' + E.Message);
      end;
    end;
    
  finally
    stlSettings.SaveToFile(sIniFile);
    stlSettings.Free;
  end;
end;

{------------------------------------------------------------------------------}
procedure QERSysMsg(const sMsg : string);
begin
{$IFDEF Q3RADIANT}
  FuncTable.pSysMsg(PChar('JKL Import: ' + sMsg + #10));
{$ENDIF}
{$IFDEF GTKRADIANT}
  FuncTable.pSysFPrintf(1, PChar('JKL Import: ' + sMsg + #10));
{$ENDIF}
end;

var
  frmStatus : TfrmStatus;

{------------------------------------------------------------------------------}
procedure CallBackStatus(iStart, iEnd : integer);
begin
  if Assigned(frmStatus) then begin
    frmStatus.bar.Max := iEnd;
    frmStatus.bar.Position := iStart;
  end;
end;

{------------------------------------------------------------------------------}
function DoImport : boolean;

  {--------------------------------------------------}
  procedure StatusMsg(const sMsg : string);
  begin
    frmStatus.lblStatus.Caption := 'Status: ' + sMsg;
    frmStatus.lblStatus.Update;
  end;

const
  OPEN_FILTER = 'All supported files (*.jkl, *.gob, *.goo)|*.jkl;*.gob;*.goo|' +
                'Jedi Knight Container files (*.gob, *.goo)|*.gob;*.goo|'+
                'Jedi Knight Level files (*.jkl)|*.jkl';

var
  sFilename : string;
  sExt : string;
  stl : TstringList;
  sItemFilename : string;
  jkl : TJKLevel;
  bsl : TBrushLevel;
  jkc : TJKContainer;
  iTex : integer;
begin
  with TOpenDialog.Create(nil) do
  try
    Options := Options + [ofFileMustExist];
    Filter := OPEN_FILTER;
    if Execute then
      sFilename := Filename;
  finally
    Free;
  end;

  frmStatus := TfrmStatus.Create(nil);
  with frmStatus do
  try
    Result := sFilename <> '';
    iTex := 0;

    if Result then begin
      Show;
      Update;

      sExt := LowerCase(ExtractFileExt(sFilename));
      jkl := TJKLevel.Create;
      bsl := TBrushLevel.Create;
      try

        if sExt = '.jkl' then begin
          StatusMsg('Parsing JKL...');
          Result := jkl.LoadJKLFromFile(sFilename, CallBackStatus);
          StatusMsg('Converting Textures...');
          iTex := ExportLevelTextures(jkl, sFilename, '', CallBackStatus);
        end
        else if (sExt = '.gob') or (sExt = '.goo') then begin
          stl := TStringList.Create;
          jkc := TJKContainer.Create;
          try
            Result := jkc.LoadFromFile(sFilename);
            if Result then begin
              sItemFilename := TfrmDlgGob.ShowDlg(jkc, 'jkl\');
              Result := sItemFilename <> '';
            end;

            if Result then begin
              StatusMsg('Extracting JKL...');
              jkc.ExtractToStl(stl, sItemFilename);
              StatusMsg('Parsing JKL...');
              jkl.LoadJKLFromStl(stl, CallBackStatus);
              jkl.Name := sItemFilename;
              StatusMsg('Converting Textures...');
              iTex := ExportLevelTextures(jkl, '', sFilename, CallBackStatus);
            end
            else
              raise Exception.Create('File selection canceled');
          finally
            jkc.Free;
            stl.Free;
          end;
        end
        else
          Result := FALSE;

        if Result then begin
          StatusMsg('Converting Sectors to Brushes...');
          Result := {SecToBrush}JklToBsl(jkl, bsl, CallBackStatus);
        end;
        if Result then begin
          StatusMsg('Loading Brushes into Radiant...');
          ExportBslToRadiant(bsl, jkl.Name + '/', CallBackStatus);
        end;

      except
        jkl.Free;
        bsl.Free;
        raise; // Move along...
      end;

      if Result then begin
        if sItemFilename = '' then
          QERSysMsg(Format('%s imported succesfully', [ExtractFileName(sFilename)]))
        else
          QERSysMsg(Format('%s imported succesfully', [ExtractFileName(sItemFilename)]));
        QERSysMsg(Format('%d sectors converted to %d brushes, %d textures converted',
          [jkl.lstWorldSectors.Count, bsl.lstWorldBrushes.Count, iTex]));
      end
      else
        QERSysMsg('Failed: Unknown reason...');

      jkl.Free;
      bsl.Free;
    end
    else
      raise Exception.Create('File selection canceled');
  finally
    Free;
  end;
end;

{------------------------------------------------------------------------------}
function DoConvTex : boolean;
begin
  Result := TRUE;
  TfrmConvTex.ShowDlg(stlSettings);
end;

{------------------------------------------------------------------------------}
function JklToBsl(jkl : TJKLevel ; bsl : TBrushLevel ;
  procStatus : TCallbackStatus) : boolean;
var
  i, j, l : integer;
  bsh : TBrush;
  sec : TSector;
  sur : TSurface;
  surTop : TSurface;
  surBot : TSurface;
  surSide : TSurface;
  pThg,
  pNewThg : PThing;
  pLit,
  pNewLit : PLight;
begin
  // for every JKL sector
  //   for every surface
  //      create brush
  //      copy surface to brush
  //      extrude new copy of surface along its negative normal
  for i := 0 to jkl.lstWorldSectors.Count - 1 do begin
    if Assigned(procStatus) then
      procStatus(i, jkl.lstWorldSectors.Count);
    sec := jkl.lstWorldSectors[i];
    for j := 0 to sec.Surfaces.Count - 1 do begin
      sur := sec.Surfaces[j];
      if sur.iAdjoin <> -1 then
        Continue;

      bsh := TBrush.Create;
      bsl.lstWorldBrushes.Add(bsh);

      // Top surface
      surTop := TSurface.Create;
      surTop.sMaterial := sur.sMaterial;

      // Obsolete...
      if (Sky  in TSurfaceFlags(sur.Flags)) or
         (Sky2 in TSurfaceFlags(sur.Flags)) then
        surTop.Flags := 4;
        
      bsh.Surfaces.Add(surTop);
      for l := sur.Vertices.Count - 1 downto 0 do
        surTop.AddVertex(
          TVertex.Create(
            sur.Vertices[l].v.x * Scale,
            sur.Vertices[l].v.y * Scale,
            sur.Vertices[l].v.z * Scale));

      // Bottom surface
      surBot := TSurface.Create;
      surBot.Flags := surTop.Flags;
      surBot.sMaterial := sur.sMaterial;
      bsh.Surfaces.Add(surBot);
      for l := surTop.Vertices.Count - 1 downto 0  do
        surBot.AddVertex(
          TVertex.Create(
            surTop.Vertices[l].v.x - (surTop.vecNormal.x * BrushDepth),
            surTop.Vertices[l].v.y - (surTop.vecNormal.y * BrushDepth),
            surTop.Vertices[l].v.z - (surTop.vecNormal.z * BrushDepth) ));

      // Side surfaces
      for l := surTop.Vertices.Count - 1 downto 0  do begin
        surSide := TSurface.Create;
        surSide.sMaterial := sur.sMaterial;
        surSide.Flags := surTop.Flags;
        bsh.Surfaces.Add(surSide);
        if l <> (surTop.Vertices.Count - 1) then begin
          surSide.Vertices.Add(surBot.Vertices[(surBot.Vertices.Count-1) - (l + 0)]);
          surSide.Vertices.Add(surBot.Vertices[(surBot.Vertices.Count-1) - (l + 1)]);
          surSide.Vertices.Add(surTop.Vertices[l + 1]);
          surSide.Vertices.Add(surTop.Vertices[l + 0]);
        end
        else begin
          // Last
          surSide.Vertices.Add(surBot.Vertices[(surBot.Vertices.Count-1) - (l + 0)]);
          surSide.Vertices.Add(surBot.Vertices[(surBot.Vertices.Count-1)]);
          surSide.Vertices.Add(surTop.Vertices[0]);
          surSide.Vertices.Add(surTop.Vertices[l + 0]);
        end;
      end;
//      vxlScale(surBot.Vertices, 0.5);
    end;
  end;

  for i := 0 to jkl.lstWorldThings.Count - 1 do begin
    New(pNewThg);
    pThg := PThing(jkl.lstWorldThings.Objects[i]);
    if LowerCase(pThg^.sName) = 'walkplayer' then
      pNewThg^.sName := 'info_player_deathmatch'
    else
      pNewThg^.sName := 'info_null';
    pNewThg^.Pos := vecMultiply(pThg^.Pos, P_vec.vec(Scale, Scale, Scale));
    pNewThg^.Rot := pThg^.Rot;
    bsl.lstWorldThings.AddObject(pNewThg^.sName, TObject(pNewThg));
  end;

  for i := 0 to jkl.lstWorldLights.Count - 1 do begin
    New(pNewLit);
    pLit := PLight(jkl.lstWorldLights[i]);
    pNewLit^.iType := pLit^.iType;
    pNewLit^.Pos := vecMultiply(pLit^.Pos, P_vec.vec(Scale, Scale, Scale));
    pNewLit^.fRange := pLit^.fRange * Scale;
    pNewLit^.fIntensity := pLit^.fIntensity * 100;
    bsl.lstWorldLights.Add(TObject(pNewLit));
  end;

  Result := TRUE;
end;

{------------------------------------------------------------------------------}
// Straight sector to brush conversion; used for test only
function SecToBrush(jkl : TJKLevel ; bsl : TBrushLevel) : boolean;
var
  i, j, v : integer;
  sec : TSector;
  sur : TSurface;
  bsh : TBrush;
  surBsh : TSurface;
  vx : TVertex;
begin
  for i := 0 to Pred(jkl.lstWorldSectors.Count) do begin
    sec := jkl.lstWorldSectors[i];
    bsh := TBrush.Create;
    bsl.lstWorldBrushes.Add(bsh);
    
    for j := 0 to Pred(sec.Surfaces.Count) do begin
      sur := sec.Surfaces[j];
      surBsh := TSurface.Create;
      bsh.Surfaces.Add(surBsh);
      for v := 0 to Pred(sur.Vertices.Count) do begin
        vx := TVertex.Create(sur.Vertices[v].v.x * 100,
                             sur.Vertices[v].v.Y * 100,
                             sur.Vertices[v].v.Z * 100);
        bsh.Vertices.Add(vx);
        surBsh.AddVertex(vx);
      end;
    end;
  end;

  Result := TRUE;
end;

{------------------------------------------------------------------------------}
function ExportLevelTextures(jkl : TJKLevel ;
  const sJKLFile, sGobFile : string ; procStatus : TCallbackStatus) : integer;
var
  sTexDir : string;
  sTexExt : string;
  sOutputFileName : string;
  sMatName : string;
  bLoaded : boolean;
  stlResGobs : TStringList;
  stlConverted : TStringList;
  jkt : TJKTexture;
  jkc : TJKContainer;
  i, iGob : integer;
  stm : TMemoryStream;
  iCount : integer;
  cmp : TColorMap;
  sPalette : string;
  iPos : integer;
begin
  Result := 0;

  // Q3Radiant plugin v1.5 dont support pGetGamePath, read from registry instead
  sTexDir := GetGamePath{ExtractFilePath(FuncTable.pGetGamePath)} + '\GameData\base\Textures\' + jkl.Name + '\';
  try
    ForceDirectories(sTexDir);
  except
    QERSysMsg('Failed to create texture directory ''' + sTexDir + ''', unable to convert textures.');
    Exit;
  end;

  // Load palette
  cmp := TColorMap.Create;
  sPalette := stlSettings.Values[INI_JKPALETTE];
  stm := TMemoryStream.Create;
  try
    iPos := Pos('>>', sPalette);
    if iPos = 0 then
      cmp.LoadPaletteFromFile(sPalette)
    else begin
      jkc := TJKContainer.Create;
      try
        jkc.LoadFromFile(Copy(sPalette, 1, iPos-1));
        jkc.ExtractToStream(stm, Copy(sPalette, iPos+2, Length(sPalette)));
        stm.Position := 0;
        cmp.LoadPaletteFromStream(stm);
      finally
        jkc.Free;
      end;
    end;
  except
    cmp.Palette := DEFAULT_PALETTE;
    QERSysMsg('Failed to load palette ''' + sPalette + ''', using default');
  end;
  stm.Free;

  sTexExt := stlSettings.Values[INI_OUTPUTTEXFORMAT];
  if sTexExt = '' then
    sTexExt := 'jpg';

  // Convert textures
  iCount := 0;
  stlResGobs := TStringList.Create;
  stlConverted := TStringList.Create;
  jkt := TJKTexture.Create;
  jkt.Palette := cmp.Palette;
  jkc := TJKContainer.Create;
  stm := TMemoryStream.Create;
  try
    stlResGobs.CommaText := sGobFile + ',' + stlSettings.Values[INI_RESOURCEGOBS];
    for i := 0 to jkl.lstUsedMaterials.Count - 1 do begin
      procStatus(i, jkl.lstUsedMaterials.Count);
      sMatName := jkl.lstUsedMaterials[i];
      sOutputFileName := sTexDir + ChangeFileExt(sMatName, '.' + sTexExt);

      if FileExists(sOutputFilename) then
        Continue; // Already converted, skip

      bLoaded := FALSE;

      // Find and load material
      if FileExists(ExtractFilePath(sJKLFile) + sMatName) then
        bLoaded := jkt.LoadFromFile(ExtractFilePath(sJKLFile) + sMatName)
      else if FileExists(ExtractFilePath(sJKLFile) + 'mat\' + sMatName) then
        bLoaded := jkt.LoadFromFile(ExtractFilePath(sJKLFile) + '\mat\' + sMatName)
      else begin // resource gobs
        for iGob := 0 to stlResGobs.Count - 1 do
          if FileExists(stlResGobs[iGob]) then begin
            jkc.LoadFromFile(stlResGobs[iGob]);
            if jkc.stlFiles.IndexOf('mat\' + sMatName) <> -1 then begin
              stm.Clear;
              stm.Position := 0;
              jkc.ExtractToStream(stm, 'mat\' + sMatName);
              bLoaded := jkt.LoadFromStream(stm);
              Break;
            end;
          end;
      end;

      if bLoaded then begin
        jkt.SaveToFile(sOutputFileName);
        stlConverted.Add(LowerCase(sMatName));
        QERSysMsg('Saved: ' + sOutputFileName);
        inc(iCount);
      end
      else
        QERSysMsg('Texture ''' + sMatName + ''' not found');
    end;
  finally
    stm.Free;
    jkc.Free;
    jkt.Free;
    cmp.Free;
    stlConverted.Free;
    stlResGobs.Free;
  end;
  Result := iCount;
end;

{------------------------------------------------------------------------------}
function ExportBslToRadiant(bsl : TBrushLevel ;
  const sTexPath : string ; procStatus : TCallbackStatus) : boolean;
var
  v1, v2, v3 : TVec3;
  i, j : integer;
  bsh : TBrush;
  pBsh : pointer;
  sur : TSurface;
  pFace : PQERFaceData;
  sNonVisTex : string;

{$IFDEF GTKRADIANT}
  pEnt : pointer;
  pEntPair : PEPair;
  pThg : PThing;
  sName : string;
  pLit : PLight;
{$ENDIF}
begin
  sNonVisTex := stlSettings.Values[INI_NONVISTEX];

  // Geometry
  for i := 0 to bsl.lstWorldBrushes.Count - 1 do begin
    if Assigned(procStatus) then
      procStatus(i, bsl.lstWorldBrushes.Count);
    bsh := bsl.lstWorldBrushes[i];
    pBsh := FuncTable.pCreateBrushHandle;

    for j := 0 to bsh.Surfaces.Count - 1 do begin
      sur := bsh.Surfaces[j];
      if sur.Vertices.Count < 3 then
        Continue; //!!

      v1 := TVec3(sur.Vertices[0].v);
      v2 := TVec3(sur.Vertices[1].v);
      v3 := TVec3(sur.vecCenter);
      FuncTable.pAddFace(pBsh, v1, v2, v3);
    end;

    for j := 0 to FuncTable.pGetFaceCount(pBsh) - 1 do begin
      sur := bsh.Surfaces[j];

      pFace := FuncTable.pGetFaceData(pBsh, j);

      if j <> FuncTable.pGetFaceCount(pBsh) - 1 then begin
        if sNonVisTex <> '' then begin
          FillChar(pFace^.TextureName, QER_MAX_NAMELEN, 0);
          StrLCopy(pFace^.TextureName,
                   PChar(sNonVisTex),
                   QER_MAX_NAMELEN-1);
        end;
      end
      else begin // The only "visible" surface
        FillChar(pFace^.TextureName, QER_MAX_NAMELEN, 0);
        StrLCopy(pFace^.TextureName,
                 PChar(sTexPath + ChangeFileExt(sur.sMaterial, '')),
                 QER_MAX_NAMELEN-1);
      end;
      pFace^.Flags := sur.Flags; // Obsolete

      FuncTable.pSetFaceData(pBsh, j, pFace);

    end;

    FuncTable.pCommitBrushHandleToMap(pBsh);

  end;

{$IFDEF GTKRADIANT}
  // Things
  for i := 0 to bsl.lstWorldThings.Count - 1 do begin
    pThg := PThing(bsl.lstWorldThings.Objects[i]);
    sName := pThg^.sName;
    if pThg^.sName = 'info_player_deathmatch' then begin

      // entity
      pEnt := FuncTable.pCreateEntityHandle;
      pEntPair := FuncTable.pAllocateEpair(PChar('classname'), PChar(sName));
      pEntPair^.next := FuncTable.pAllocateEpair(PChar('angle'), PChar(IntToStr(Trunc(pThg^.rot.y + 90))));
      pEntPair^.next^.next :=
        FuncTable.pAllocateEpair(PChar('origin'),
                                 PChar(IntToStr(Trunc(pThg^.pos.x)) + ' ' +
                                       IntToStr(Trunc(pThg^.pos.y)) + ' ' +
                                       IntToStr(Trunc(pThg^.pos.z))));
      FuncTable.pSetEntityKeyValList(pEnt, pEntPair);

      FuncTable.pCommitEntityHandleToMap(pEnt);
    end;
  end;

  // Lights
  for i := 0 to bsl.lstWorldLights.Count - 1 do begin
    pLit := PLight(bsl.lstWorldLights[i]);

    pEnt := FuncTable.pCreateEntityHandle;
    pEntPair := FuncTable.pAllocateEpair(PChar('classname'), PChar('light'));
    pEntPair^.next :=
      FuncTable.pAllocateEpair(PChar('light'), PChar(FloatToStr(pLit^.fIntensity)));
    pEntPair^.next^.next :=
      FuncTable.pAllocateEpair(PChar('radius'), PChar(FloatToStr(pLit^.fRange)));
    pEntPair^.next^.next^.next :=
      FuncTable.pAllocateEpair(PChar('origin'),
                                   PChar(IntToStr(Trunc(pLit^.pos.x)) + ' ' +
                                         IntToStr(Trunc(pLit^.pos.y)) + ' ' +
                                         IntToStr(Trunc(pLit^.pos.z))));
    FuncTable.pSetEntityKeyValList(pEnt, pEntPair);

    FuncTable.pCommitEntityHandleToMap(pEnt);
  end;
{$ENDIF}

  Result := TRUE;
end;

{------------------------------------------------------------------------------}
function Scale : double;
const
  SCALE_CACHE : double = 0;
begin
  if SCALE_CACHE = 0 then
    try
      SCALE_CACHE := StrToFloat(stlSettings.Values[INI_SCALE]);
    except
      SCALE_CACHE := DEF_SCALE;
      QERSysMsg(
        'Error reading scale factor setting, using default: ' + FloatToStr(DEF_SCALE));
    end;
  Result := SCALE_CACHE;
end;

{------------------------------------------------------------------------------}
function BrushDepth : double;
const
  BRUSHDEPTH_CACHE : double = 0;
begin
  if BRUSHDEPTH_CACHE = 0 then
    try
      BRUSHDEPTH_CACHE := StrToFloat(stlSettings.Values[INI_BRUSHDEPTH]);
    except
      BRUSHDEPTH_CACHE := 8;
      QERSysMsg(
        'Error reading brush depth setting, using default: 8');
    end;
  Result := BRUSHDEPTH_CACHE;
end;

{------------------------------------------------------------------------------}
function GetGamePath : string;
begin
// 'HKEY_LOCAL_MACHINE\SOFTWARE\LucasArts Entertainment Company LLC\Star Wars JK II Jedi Outcast\1.0'
  Result := '';
  with TRegistry.Create(KEY_READ	) do
  try
    RootKey := HKEY_LOCAL_MACHINE;
    if OpenKey('Software\LucasArts Entertainment Company LLC\Star Wars JK II Jedi Outcast\1.0\', FALSE) then begin
      Result := ReadString('Install Path');
      CloseKey;
    end;
  finally
    Free;
  end;
end;

end.
